<?php

namespace App\Factory;

use App\Exception;

class BranchFactory {
    
    private $connection;
    
    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function getStandartClient ($branch) {
        $sql = "
            SELECT CDCLIENTE
              FROM PARAVEND
            WHERE CDFILIAL = :CDFILIAL";
            
        $params = array(
            ":CDFILIAL" => $branch
        );
        
        $stdClient = $this->connection->fetchAll($sql, $params)[0];
        
        if (empty($stdClient["CDCLIENTE"])) throw new Exception\ParamsException("Não foi encontrado cliente padrão para a filial - $branch", 105);
        
        return $stdClient;
    }
    
    public function getStandartConsumerType ($branch) {
        $sql = "
            SELECT CDTIPOCONS, CDCCUSCLIE
              FROM PARAVEND
            WHERE CDFILIAL = :CDFILIAL";
            
        $params = array(
            ":CDFILIAL" => $branch    
        );
        
        $consumerT = $this->connection->fetchAll($sql, $params)[0];
        
        if (empty($consumerT["CDTIPOCONS"]) || empty($consumerT["CDCCUSCLIE"]))
            throw new Exception\ParamsException("Não foi encontrado tipo de consumidor padrão para a filial - $branch", 106);
        
        return $consumerT;
    }
    
    public function getAddressInfo ($branch) {
        $sql = "
            SELECT CDMUNICIPIO, SGESTADO, DSENDEFILI, NMBAIRFILI, NRCEPFILI
              FROM ENDEFILI
            WHERE CDFILIAL = :CDFILIAL
              AND IDTPENDEFILI = 'P'";
        
        $params = array(
            ":CDFILIAL" => $branch    
        );
        
        return $this->connection->fetchAll($sql, $params)[0];
    }
    
    public function getBranchCityCode($branch) {
        $sql = "
            SELECT CDMUNICIPIO
              FROM ENDEFILI
             WHERE CDFILIAL = :CDFILIAL";
             
        $params = array(
            ":CDFILIAL" => $branch
        );
        
        return $this->connection->fetchAll($sql, $params)[0]["CDMUNICIPIO"];
    }
}
