<?php

namespace App\Factory;

use App\Exception;

class OrderFactory {

    private $connection;

    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function insertSale ($sale, $generalParams, $consumerDetails) {
        $sql = "
            INSERT INTO VENDAREST
    			(CDFILIAL, NRVENDAREST, CDLOJA, NRMESA, CDVENDEDOR,
    			DTHRABERMESA, DTHRFECHMESA, CDOPERADOR, NRPESMESAVEN,
    			CDCLIENTE, CDCONSUMIDOR, IDPEDIDOPAGO)
    		VALUES
    			(:CDFILIAL, :NRVENDAREST, :CDLOJA, :NRMESA,
    			:CDVENDEDOR, TO_DATE(:CURRDATE, 'dd/mm/yyyy hh24:mi:ss'), null, :CDOPERADOR,
    			:NRPESMESAVEN, :CDCLIENTE, :CDCONSUMIDOR, 'S')";
    			
    	$params = array(
    	    ":CDFILIAL"     => $generalParams["CDFILIAL"],
            ":NRVENDAREST"  => $sale,
            ":CDLOJA"       => $generalParams["CDLOJA"],
            ":NRMESA"       => "",
            ":CDVENDEDOR"   => $generalParams["CDVENDEDOR"],
            ":CURRDATE"     => date("d/m/Y H:i:s"),
            ":CDOPERADOR"   => $generalParams["CDOPERADOR"],
            ":NRPESMESAVEN" => "1",
            ":CDCLIENTE"    => $consumerDetails["CDCLIENTE"],
            ":CDCONSUMIDOR" => $consumerDetails["CDCONSUMIDOR"]
    	);
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function insertSaleCommand ($orderData, $generalParams, $consumerDetails) {
        $sql = "
            INSERT INTO COMANDAVEN 
                (CDFILIAL, NRVENDAREST, NRCOMANDA, NRCOMANDAEXT, 
    	        CDLOJA, DSCOMANDA, IDSTCOMANDA, SGSEXOCON, 
    	        TXMOTIVCANCE, VRACRCOMANDA, CDPAIS, SGESTADO, 
    	        CDMUNICIPIO, NRCEPCONSCOMAND, CDBAIRRO, DSBAIRRO, 
    	        DSENDECONSCOMAN, NRENDECONSCOMAN, DSCOMPLENDCOCOM,
    	        DSREFENDCONSCOM, DSOBSCOMANDA, IDORGCMDVENDA, IDRETBALLOJA,
    	        NRPEDMOBILE, NRLATPEDIDO, NRLONGPEDIDO, DSCONSUMIDOR, NRCONTATOCON)
    	    VALUES 
    	        (:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRCOMANDAEXT, 
    	        :CDLOJA, :DSCOMANDA, :IDSTCOMANDA, :SGSEXOCON, 
    	        :TXMOTIVCANCE, :VRACRCOMANDA, :CDPAIS, :SGESTADO, 
    	        :CDMUNICIPIO, :NRCEPCONSCOMAND, :CDBAIRRO, :DSBAIRRO, 
    	        :DSENDECONSCOMAN, :NRENDECONSCOMAN, :DSCOMPLENDCOCOM,
    	        :DSREFENDCONSCOM, :DSOBSCOMANDA, :IDORGCMDVENDA,
    	        :IDRETBALLOJA, :NRPEDMOBILE, :NRLATPEDIDO, :NRLONGPEDIDO,
    	        :DSCONSUMIDOR, :NRCONTATOCON)";
    	        
    	$consumerName = strtoupper($this->removeAcentos($orderData["name"]));

    	$params = array(
    	    ":CDFILIAL"        => $generalParams["CDFILIAL"],
            ":NRVENDAREST"     => $orderData["NRVENDAREST"],
            ":NRCOMANDA"       => $orderData["NRCOMANDA"],
            ":NRCOMANDAEXT"    => $orderData["order_number"],
            ":CDLOJA"          => $generalParams["CDLOJA"],
            ":DSCOMANDA"       => "DLV_".$orderData["NRCOMANDA"],
            ":IDSTCOMANDA"     => $orderData["IDSTCOMANDA"],
            ":SGSEXOCON"       => "M",
            ":TXMOTIVCANCE"    => "",
            ":VRACRCOMANDA"    => str_replace(".", ",", $orderData["tax"]),
            ":CDPAIS"          => "0055",
            ":SGESTADO"        => $consumerDetails["SGESTADO"], 
            ":CDMUNICIPIO"     => $consumerDetails["CDMUNICIPIO"],
            ":NRCEPCONSCOMAND" => $consumerDetails["cep"],
            ":CDBAIRRO"        => $consumerDetails["CDBAIRRO"],
            ":DSBAIRRO"        => substr($consumerDetails["neighborhood"], 0, 100),
            ":DSENDECONSCOMAN" => substr($consumerDetails["street"], 0, 50),
            ":NRENDECONSCOMAN" => empty($consumerDetails["number"]) ? "" : substr($consumerDetails["number"], 0, 10),
            ":DSCOMPLENDCOCOM" => $consumerDetails["complement"],
            ":DSREFENDCONSCOM" => $consumerDetails["reference_point"],
            ":DSOBSCOMANDA"    => $orderData["notes"],
            ":IDORGCMDVENDA"   => $orderData["origin"],
            ":IDRETBALLOJA"    => $orderData["forma_entrega"] == 1 ? "N" : "S",
            ":NRPEDMOBILE"     => 'NMO' . $orderData["id"],
            ":NRLATPEDIDO"     => $orderData["latitude"],
            ":NRLONGPEDIDO"    => $orderData["longitude"],
            ":DSCONSUMIDOR"    => $consumerName,
            ":NRCONTATOCON"    => substr((isset($orderData["phone"]) ? $orderData["phone"] : ""), 0, 32),
    	);
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function insertCashMoviment ($orderData, $generalParams, $consumerDetails) {
        $sql = "
            INSERT INTO MOVCAIXADLV
        		(CDFILIAL, NRVENDAREST, NRSEQMOVDLV, IDTIPOMOVIVEDLV,
        		VRMOVIVENDDLV, CDTIPORECE, CDCLIENTE, CDCONSUMIDOR,
        		NRCARTBANCODLV)
        	VALUES
        		(:CDFILIAL, :NRVENDAREST, :NRSEQMOVDLV, :IDTIPOMOVIVEDLV,
        		:VRMOVIVENDDLV, :CDTIPORECE, :CDCLIENTE, :CDCONSUMIDOR,
        		:NRCARTBANCODLV)";
        
        $params = array(
            ":CDFILIAL"        => $generalParams["CDFILIAL"],
	        ":NRVENDAREST"     => $orderData["NRVENDAREST"],
	        ":NRSEQMOVDLV"     => $orderData["NRSEQMOVDLV"],
	        ":IDTIPOMOVIVEDLV" => "E",
	        ":CDCLIENTE"       => $consumerDetails["CDCLIENTE"],
	        ":CDCONSUMIDOR"    => $consumerDetails["CDCONSUMIDOR"],
	        ":CDTIPORECE"      => $orderData["CDTIPORECE"],
	        ":VRMOVIVENDDLV"   => str_replace(".", ",", $orderData["total"]),
	        ":NRCARTBANCODLV"  => null
        );
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function insertChange ($orderData, $generalParams, $consumerDetails, $movCaixa) {
        $value = $orderData["troco"] - $orderData["total"];
        
        $sql = "
            INSERT INTO MOVCAIXADLV
        		(CDFILIAL, NRVENDAREST, NRSEQMOVDLV, IDTIPOMOVIVEDLV,
        		VRMOVIVENDDLV, CDTIPORECE, CDCLIENTE, CDCONSUMIDOR,
        		NRCARTBANCODLV)
        	VALUES
        		(:CDFILIAL, :NRVENDAREST, :NRSEQMOVDLV, :IDTIPOMOVIVEDLV,
        		:VRMOVIVENDDLV, :CDTIPORECE, :CDCLIENTE, :CDCONSUMIDOR,
        		:NRCARTBANCODLV)";
        
        $params = array(
            ":CDFILIAL"        => $generalParams["CDFILIAL"],
	        ":NRVENDAREST"     => $orderData["NRVENDAREST"],
	        ":NRSEQMOVDLV"     => $movCaixa,
	        ":IDTIPOMOVIVEDLV" => "S",
	        ":CDCLIENTE"       => $consumerDetails["CDCLIENTE"],
	        ":CDCONSUMIDOR"    => $consumerDetails["CDCONSUMIDOR"],
	        ":CDTIPORECE"      => $orderData["CDTIPORECE"],
	        ":VRMOVIVENDDLV"   => str_replace(".", ",", $value),
	        ":NRCARTBANCODLV"  => null
        );
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function checkForChange ($orderData) {
        $total = $orderData["total"];
        
        if (!empty($orderData["troco"]))
            $total = $orderData["troco"] - $orderData["total"] > 0 ? $orderData["troco"] : $orderData["total"];
        
        return $total;
    }
    
    public function validateOperator($generalParams){
        $sql = "
            SELECT * FROM FILIOPER
             WHERE CDOPERADOR   = :CDOPERADOR
               AND CDFILIAL     = :CDFILIAL
        ";

        $params = array(
            ":CDFILIAL"   => $generalParams["CDFILIAL"],
            ":CDOPERADOR" => $generalParams["CDOPERADOR"]
        );
        
        return $this->connection->fetchAssoc($sql, $params);
        
    }
    
    public function fetchIsCashierSinc($filial){
        $sql = "
        SELECT  IDSINCCAIXADLV 
           FROM  CAIXA
           WHERE CDFILIAL = :CDFILIAL AND
                 UPPER(IDSINCCAIXADLV) = 'S'
        ";
        $params = array(
            ":CDFILIAL" => $filial
        );
        return $this->connection->fetchAssoc($sql,$params);
        
    }
    
    //Prevents teknisa from crazy discount logics by partners ¯\_(ツ)_/¯
    public function validateOrder($cdFilial, $nrVendaRest) {

        $taxSQL = "
            SELECT VRACRCOMANDA
              FROM COMANDAVEN
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
        ";

        $itemsSQL = "
            SELECT QTPRODCOMVEN, VRPRECCOMVEN, VRDESCCOMVEN
              FROM ITCOMANDAVEN
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
        ";

        $entradasSQL = "
            SELECT VRMOVIVENDDLV FROM MOVCAIXADLV
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
               AND IDTIPOMOVIVEDLV  = 'E'
        ";

        $saidasSQL = "
            SELECT VRMOVIVENDDLV FROM MOVCAIXADLV
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
               AND IDTIPOMOVIVEDLV  = 'S'
        ";
        
        $params = [
            ":CDFILIAL"     => $cdFilial,
            ":NRVENDAREST"  => $nrVendaRest
        ];

        $taxData      = $this->connection->fetchAssoc($taxSQL, $params);
        $itemsData    = $this->connection->fetchAll($itemsSQL, $params);
        $entradasData = $this->connection->fetchAll($entradasSQL, $params);
        $saidasData   = $this->connection->fetchAll($saidasSQL, $params);

        $taxSum      = isset($taxData['VRACRCOMANDA']) ? (string)$this->formatNumber($taxData['VRACRCOMANDA']) : "0.00";
        $itemsSum    = "0.00";
        $entradasSum = "0.00";
        $saidasSum   = "0.00";

        foreach($itemsData as $itemData) {
            $mul      = bcmul($this->formatNumber($itemData['VRPRECCOMVEN']), $this->formatNumber($itemData['QTPRODCOMVEN']), 2);
            $sub      = bcsub($mul, $this->formatNumber($itemData['VRDESCCOMVEN']), 2);
            $itemsSum = bcadd($itemsSum, $sub, 2);
        }

        foreach($entradasData as $entradaData) {
            $entradasSum = bcadd($entradasSum, $this->formatNumber($entradaData['VRMOVIVENDDLV']), 2);
        }

        foreach($saidasData as $saidaData) {
            $saidasSum = bcadd($saidasSum, $this->formatNumber($saidaData['VRMOVIVENDDLV']), 2);
        }

        $orderSum    = bcadd($itemsSum, $taxSum, 2);
        $paymentsSum = bcsub($entradasSum, $saidasSum, 2);
        
        if(bccomp($orderSum, $paymentsSum) != 0) {
            throw new Exception\ParamsException("
                Não foi possível processar o pedido. Existe uma divergencia entre os itens do pedido e o valor pago.<br>
                Valor dos itens: {$orderSum}.<br>
                Valor pago: {$paymentsSum}.");
        }
    }
    
    private function formatNumber($number) {
        return str_replace(",", ".", $number);
    }
    
    public function isDuplicated($generalParams, $orderData) {
        $sql = "
            SELECT C.NRPEDMOBILE

    		  FROM COMANDAVEN C

    		  JOIN VENDAREST V
    		    ON V.NRVENDAREST   = C.NRVENDAREST
    		   AND V.CDFILIAL      = C.CDFILIAL

    		 WHERE C.NRPEDMOBILE   = :NRPEDMOBILE
    		   AND V.CDFILIAL      = :CDFILIAL
    		   AND DTHRABERMESA    >= SYSDATE - 1
        ";
        
        $params = [
            ":CDFILIAL"    => $generalParams["CDFILIAL"],
            ":NRPEDMOBILE" => "NMO".$orderData["id"]
        ];
        
        $result = $this->connection->fetchAll($sql, $params);
        
        return !empty($result);
    }
    
    private function removeAcentos($texto) {
        return preg_replace(array("/(á|à|ã|â|ä)/","/(Á|À|Ã|Â|Ä)/","/(é|è|ê|ë)/","/(É|È|Ê|Ë)/","/(í|ì|î|ï)/","/(Í|Ì|Î|Ï)/","/(ó|ò|õ|ô|ö)/","/(Ó|Ò|Õ|Ô|Ö)/","/(ú|ù|û|ü)/","/(Ú|Ù|Û|Ü)/","/(ñ)/","/(Ñ)/","/(ç)/","/(Ç)/"),explode(" ","a A e E i I o O u U n N c C"),$texto);
    }
}