<?php

namespace App\Factory;

class ProductFactory {

    private $connection;

    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function getProduct ($product) {
        $sql = "
            SELECT CDPRODUTO, CDARVPROD, NMPRODUTO, IDTIPOCOMPPROD
              FROM PRODUTO PR
             WHERE CDPRODUTO  = :CDPRODUTO
                OR CDPRODINTE = :CDPRODINTE
                OR CDARVPROD  = :CDARVPROD
        ";

        $productCode = str_replace(".", "", $product);
        $productCode = preg_replace("/[^0-9A-Z]/", "", $productCode);

        $params = array(
            ':CDPRODUTO'  => $productCode,
            ':CDPRODINTE' => $productCode,
            ":CDARVPROD"  => $productCode
        );
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function insertProduct ($product, $generalParams, $orderData) {
        $sql = "
            INSERT INTO ITCOMANDAVEN
    			(CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN,
    			CDPRODUTO, QTPRODCOMVEN, VRPRECCOMVEN, TXPRODCOMVEN,
    			IDSTPRCOMVEN, VRDESCCOMVEN, NRLUGARMESA, DTHRINCOMVEN,
    			IDPRODIMPFIS, CDLOJA, VRACRCOMVEN, NRSEQPRODCOM, NRSEQPRODCUP,
    			VRPRECCLCOMVEN, CDCAIXACOLETOR, CDPRODPROMOCAO, CDVENDEDOR, NRINSCRCONS, DSOBSPEDDIGCMD)
    		VALUES
    			(:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN,
    			:CDPRODUTO, :QTPRODCOMVEN, :VRPRECCOMVEN, :TXPRODCOMVEN,
    			:IDSTPRCOMVEN, :VRDESCCOMVEN, :NRLUGARMESA, TO_DATE(:CURRDATE, 'dd/mm/yyyy hh24:mi:ss'),
    			:IDPRODIMPFIS, :CDLOJA, :VRACRCOMVEN, :NRSEQPRODCOM, :NRSEQPRODCUP,
    			:VRPRECCLCOMVEN, :CDCAIXACOLETOR, :CDPRODPROMOCAO, :CDVENDEDOR, :NRINSCRCONS, :DSOBSPEDDIGCMD)";
    		
    	$params = array(
    	    ":CDFILIAL"       => $product["CDFILIAL"],
			":NRVENDAREST"    => $product["NRVENDAREST"],
			":NRCOMANDA"      => $product["NRCOMANDA"],
			":NRPRODCOMVEN"   => $product["NRPRODCOMVEN"],
			":CDPRODUTO"      => $product["CDPRODUTO"],
			":QTPRODCOMVEN"   => $product["QTPRODCOMVEN"],
			":VRPRECCOMVEN"   => str_replace(".", ",", $product["VRPRECCOMVEN"]),
			":TXPRODCOMVEN"   => $product["TXPRODCOMVEN"],
			":IDSTPRCOMVEN"   => $product["IDSTPRCOMVEN"],
			":VRDESCCOMVEN"   => str_replace(".", ",", $product["VRDESCCOMVEN"]),
			":NRLUGARMESA"    => $product["NRLUGARMESA"],
			":CURRDATE"       => date("d/m/Y H:i:s"),
			":IDPRODIMPFIS"   => $product["IDPRODIMPFIS"],
			":CDLOJA"         => $product["CDLOJA"],
			":VRACRCOMVEN"    => str_replace(".", ",", $product["VRACRCOMVEN"]),
			":NRSEQPRODCOM"   => $product["NRSEQPRODCOM"],
			":NRSEQPRODCUP"   => $product["NRSEQPRODCUP"],
			":VRPRECCLCOMVEN" => str_replace(".", ",", $product["VRPRECCLCOMVEN"]),
			":CDCAIXACOLETOR" => $product["CDCAIXACOLETOR"],
			":CDPRODPROMOCAO" => $product["CDPRODPROMOCAO"],
			":CDVENDEDOR"     => $product["CDVENDEDOR"],
			":NRINSCRCONS"    => $product["NRINSCRCONS"],
			":DSOBSPEDDIGCMD" => $product["TXPRODCOMVEN"],
			":CDSENHAPED" => $orderData["order_number"]
    	);
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function getObservation ($observation) {
    	$sql = "
    		SELECT *
    		FROM OCORRENCIA
    		WHERE CDGRPOCOR = :CDGRPOCOR
    		  AND CDOCORR = :CDOCORR";
    	
    	$obs = explode("|", $observation);
    	
    	$params = array(
    		":CDGRPOCOR" => $obs[0],
    		":CDOCORR"   => $obs[1]
    	);
    	
    	return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function insertObservation ($obs) {
        $sql = "
            INSERT INTO	OBSITCOMANDAVEN
                (CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN, CDOCORR, CDGRPOCOR, NRPRODCOMVENOBS, QTPRODCOMVENOBS)
            VALUES
                (:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN, :CDOCORR, :CDGRPOCOR, :NRPRODCOMVENOBS, :QTPRODCOMVENOBS)";
		
	    $params = array(
	        ":CDFILIAL"        => $obs["CDFILIAL"],
            ":NRVENDAREST"     => $obs["NRVENDAREST"],
            ":NRCOMANDA"       => $obs["NRCOMANDA"],
            ":NRPRODCOMVEN"    => $obs["NRPRODCOMVEN"],
            ":NRPRODCOMVENOBS" => $obs["NRPRODCOMVENOBS"],
            ":CDOCORR"         => $obs["CDOCORR"],
            ":CDGRPOCOR"       => $obs["CDGRPOCOR"],
            ":QTPRODCOMVENOBS" => $obs["QTPRODCOMVENOBS"]  
        );
        
        $this->connection->executeQuery($sql, $params);
    }
    
}