<?php

namespace App\Util;

use App\Exception;

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception as MailerException;

class Util {
    private $connection;
    
    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function newCode ($counterName, $size) {
        /* NOVOCODIGO */
        try {
            $nrseq = null;
            $connection = $this->connection;
            $storedProcedure = new StoredProcedure($connection, "NOVO_CODIGO_NT");
            $storedProcedure->addParam(new Param("P_CONTADOR", Param::PARAM_INPUT, $counterName, Param::PARAM_TYPE_STR, strlen($counterName)));
            $storedProcedure->addParam(new Param("P_SEQUENCIAL", Param::PARAM_OUTPUT, $nrseq, Param::PARAM_TYPE_INT, 20));
            $result = $storedProcedure->execute();
            return substr($result["P_SEQUENCIAL"], strlen($result["P_SEQUENCIAL"])-$size);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    /**
     *
     *
     * @param StoredProcedure $procedure The procedure to be executed.
     * @param array $paramValues Param values to override values present in Param instance.
     *
     * @return array The output parameters indexed by name.
     */
    public function executeProcedure(StoredProcedure $procedure, array $paramValues = array()) {
        $queryStmt = $this->getQueryString($procedure);
        $stmt = $this->connection->prepare($queryStmt);
        $valuesOutput = array();
        foreach ($procedure->getParams() as $param) {
            if ($param->isOutput()) {
                $valuesOutput[$param->getName()] = $param->getValue();
                $stmt->bindParam($param->getParamAlias(), $valuesOutput[$param->getName()], $param->getType(), $param->getLength());
            } else {
                $stmt->bindValue($param->getParamAlias(), $param->getValue(), $param->getType());
            }
        }

        $stmt->execute();
        return $valuesOutput;
    }

    /**
     * @param StoredProcedure $procedure
     * @return string
     */
    protected function getQueryString(StoredProcedure $procedure) {
        $paramNames = array_map(function (Param $param) {
            return ":" . $param->getName();
        }, $procedure->getParams());
        $paramString = implode(", ", $paramNames);
        return "CALL " . $procedure->getName() . " ( " . $paramString . " )";
    }
    
    public function sendMail($recipient, $cc, $subject, $body, $params) {
        try {
            $configFile = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config     = json_decode($configFile, true);
        
            $mail = new PHPMailer(true);

            $mail->isSMTP();                            
            $mail->Host       = $config["host"];          
            $mail->SMTPAuth   = true;                   
            $mail->Username   = $config["username"];
            $mail->Password   = $this->decrypt($config["password"]); 
            $mail->SMTPSecure = $config["SMTPSecure"];    
            $mail->Port       = $config["port"];      
            $mail->CharSet    = "UTF-8";
            $mail->Encoding   = "base64";
            
            $mail->setFrom($config["from"], "Teknisa Retail");

            if(empty($cc)) {
                $cc = ["suporte.odhen@teknisa.com"];
                // $cc = ["daniel.fonseca@teknisa.com"];
            }

            foreach ($recipient as $r) { $mail->addAddress($r); }
            foreach ($cc as $c) { $mail->addCC($c); }
            
            $mail->isHTML(true);                                  
            $mail->Subject = $subject;
            $mail->Body    = str_replace(array_keys($params), array_values($params), $body);
            
            $mail->send();
        } catch (Exception $e) {
            var_dump("error: ".$e->getMessage());
        }
    
    }
    
    public function getPendingOrdersFile() {
        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);

        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";
        
        $basePath     = $config['logPath'] . "/$clientFolder";

        $folder = $basePath."/dispatch";

        $fileName = "/pendingOrders.json";
        $currentFile = $folder . $fileName;
        
        if(!file_exists($folder)){
            mkdir($folder, 0775, true);
            file_put_contents($currentFile, json_encode([]));
        }
        
        if(!file_exists($currentFile)){
            file_put_contents($currentFile, json_encode([]));
        }

        return $currentFile;
    }
    
    public function getCurrentTimezone($state) {
        $timezones = [
            "AC" => "America/Rio_branco",   "AL" => "America/Maceio",
            "AP" => "America/Belem",        "AM" => "America/Manaus",
            "BA" => "America/Bahia",        "CE" => "America/Fortaleza",
            "DF" => "America/Sao_Paulo",    "ES" => "America/Sao_Paulo",
            "GO" => "America/Sao_Paulo",    "MA" => "America/Fortaleza",
            "MT" => "America/Cuiaba",       "MS" => "America/Campo_Grande",
            "MG" => "America/Sao_Paulo",    "PR" => "America/Sao_Paulo",
            "PB" => "America/Fortaleza",    "PA" => "America/Belem",
            "PE" => "America/Recife",       "PI" => "America/Fortaleza",
            "RJ" => "America/Sao_Paulo",    "RN" => "America/Fortaleza",
            "RS" => "America/Sao_Paulo",    "RO" => "America/Porto_Velho",
            "RR" => "America/Boa_Vista",    "SC" => "America/Sao_Paulo",
            "SE" => "America/Maceio",       "SP" => "America/Sao_Paulo",
            "TO" => "America/Araguaia",    
        ];
       
        return $timezones[$state];
    }

    public function writeLog($type, $fileName, $content) {

        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);

        $clientFolder = $this->connection->getParams();

        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";

        $basePath     = $config['logPath'] . "/$clientFolder";

        if($type == 'order') {
            $folder      = $basePath . "/orders/" . date("Y") . "/" . date("m") . "/" . date("d");
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'order_processed') {
            $folder      = $basePath . "/processed/" . date("Y") . "/" . date("m") . "/" . date("d");
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'order_error') {
            $folder      = $basePath . "/error/" . date("Y") . "/" . date("m") . "/" . date("d");

            $currentFile = "$folder/$fileName";

            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }
            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }
        
        if($type == 'network') {
            $folder      = $basePath . "/networkError/" . date("Y") . "/" . date("m") . "/" . date("d");
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content);
        }

        if($type == 'execution') {
            $folder      = $basePath . "/execution/" . date("Y") . "/" . date("m") . "/";

            $currentFile = "$folder" . date("d") . ".txt";

            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content, FILE_APPEND);
        }

    }
    
    public function getMailList() {
        $list = [];

        $sql = "
            SELECT DSEMAILTEC 
              FROM LISTAEMAILTEC
              WHERE CDPROJETO = '009'
        ";
        
        $result = $this->connection->fetchAll($sql);
        
        if(empty($result)) {
            throw new Exception\GeneralErrorException("Cadastro de responsáveis técnicos inexistente.");
        }
        
        foreach($result as $row) {
            if(!empty($row['DSEMAILTEC']) && filter_var($row['DSEMAILTEC'], FILTER_VALIDATE_EMAIL)) {
                array_push($list, $row['DSEMAILTEC']);
            }
        }
        
        if(empty($list)) {
            throw new Exception\GeneralErrorException("Cadastro de responsáveis técnicos inexistente.");
        }
        
        return $list;
    }
    
    private function decrypt($string) {
        $output = false;
        $encrypt_method = "AES-256-CBC";
    
        $secret_key = 'ZEEDHI';
        $secret_iv  = 'ZEEDHI IV';
    
        $key = hash('sha256', $secret_key);
        $iv  = substr(hash('sha256', $secret_iv), 0, 16);

        $output = openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
        return $output;
    }
}